<?php
/* --------------------------------------------------------------
   CustomerGroupWriter.inc.php 2017-09-06
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class CustomerGroupWriter
 *
 * @category   System
 * @package    CustomerGroup
 * @subpackage Repositories
 */
class CustomerGroupWriter implements CustomerGroupWriterInterface
{
	/**
	 * @var \CI_DB_query_builder
	 */
	protected $queryBuilder;
	
	/**
	 * @var \LanguageProvider
	 */
	protected $languageProvider;
	
	
	/**
	 * CustomerGroupWriter constructor.
	 *
	 * @param \CI_DB_query_builder $queryBuilder
	 * @param \LanguageProvider    $languageProvider
	 */
	public function __construct(CI_DB_query_builder $queryBuilder, LanguageProvider $languageProvider)
	{
		$this->queryBuilder     = $queryBuilder;
		$this->languageProvider = $languageProvider;
	}
	
	
	/**
	 * Saves customer group entity data in database.
	 *
	 * @param \CustomerGroupInterface $customerGroup Customer group to be saved.
	 *
	 * @return $this|\CustomerGroupWriterInterface Same instance for chained method calls.
	 */
	public function store(CustomerGroupInterface $customerGroup)
	{
		$lastCustomerGroupId = $this->queryBuilder->select('customers_status_id')
		                                          ->from('customers_status')
		                                          ->order_by('customers_status_id', 'DESC')
		                                          ->get()
		                                          ->row_array()['customers_status_id'];
		
		$customerGroupId = (int)$lastCustomerGroupId + 1;
		
		if(count($customerGroup->getNames()) > 0)
		{
			foreach($customerGroup->getNames() as $languageCode => $name)
			{
				$this->queryBuilder->set([
					                         'customers_status_id'                  => $customerGroupId,
					                         'language_id'                          => $this->languageProvider->getIdByCode(MainFactory::create('LanguageCode',
					                                                                                                                            new StringType($languageCode))),
					                         'customers_status_name'                => $name,
					                         'customers_status_public'              => $customerGroup->getSettings()
					                                                                                 ->isPublic(),
					                         'customers_status_min_order'           => $customerGroup->getConfigurations()
					                                                                                 ->getMinOrder(),
					                         'customers_status_max_order'           => $customerGroup->getConfigurations()
					                                                                                 ->getMaxOrder(),
					                         'customers_status_discount'            => $customerGroup->getConfigurations()
					                                                                                 ->getDiscount(),
					                         'customers_status_ot_discount_flag'    => $customerGroup->getSettings()
					                                                                                 ->isOtDiscountFlag(),
					                         'customers_status_ot_discount'         => $customerGroup->getConfigurations()
					                                                                                 ->getOtDiscount(),
					                         'customers_status_graduated_prices'    => $customerGroup->getSettings()
					                                                                                 ->isGraduatedPrices(),
					                         'customers_status_show_price'          => $customerGroup->getSettings()
					                                                                                 ->isShowPrice(),
					                         'customers_status_show_price_tax'      => $customerGroup->getSettings()
					                                                                                 ->isShowPriceTax(),
					                         'customers_status_add_tax_ot'          => $customerGroup->getSettings()
					                                                                                 ->isAddTaxOt(),
					                         'customers_status_payment_unallowed'   => implode(',',$customerGroup->getConfigurations()
					                                                                                 ->getUnallowedPaymentModules()),
					                         'customers_status_shipping_unallowed'  => implode(',',$customerGroup->getConfigurations()
					                                                                                 ->getUnallowedShippingModules()),
					                         'customers_status_discount_attributes' => $customerGroup->getSettings()
					                                                                                 ->isDiscountAttributes(),
					                         'customers_fsk18'                      => $customerGroup->getSettings()
					                                                                                 ->isFsk18(),
					                         'customers_fsk18_display'              => $customerGroup->getSettings()
					                                                                                 ->isFsk18Display(),
					                         'customers_status_write_reviews'       => $customerGroup->getSettings()
					                                                                                 ->isWriteReviews(),
					                         'customers_status_read_reviews'        => $customerGroup->getSettings()
					                                                                                 ->isReadReviews()
				                         ]);
				
				$this->queryBuilder->insert('customers_status');
			}
		}
		
		$customerGroup->setId(new IntType($customerGroupId));
		
		return $this;
	}
	
	
	/**
	 * Updates customer group entity data in database.
	 *
	 * @param \CustomerGroupInterface $customerGroup Customer group to be  updated.
	 *
	 * @return $this|\CustomerGroupWriterInterface Same instance for chained method calls.
	 */
	public function update(CustomerGroupInterface $customerGroup)
	{
		foreach($customerGroup->getNames() as $languageCode => $name)
		{
			$languageId = $this->languageProvider->getIdByCode(new LanguageCode(new StringType($languageCode)));
			
			$this->queryBuilder->update('customers_status', [
				'customers_status_name'                => $name,
				'customers_status_public'              => $customerGroup->getSettings()->isPublic(),
				'customers_status_min_order'           => $customerGroup->getConfigurations()->getMinOrder(),
				'customers_status_max_order'           => $customerGroup->getConfigurations()->getMaxOrder(),
				'customers_status_discount'            => $customerGroup->getConfigurations()->getDiscount(),
				'customers_status_ot_discount_flag'    => $customerGroup->getSettings()->isOtDiscountFlag(),
				'customers_status_ot_discount'         => $customerGroup->getConfigurations()->getOtDiscount(),
				'customers_status_graduated_prices'    => $customerGroup->getSettings()->isGraduatedPrices(),
				'customers_status_show_price'          => $customerGroup->getSettings()->isShowPrice(),
				'customers_status_show_price_tax'      => $customerGroup->getSettings()->isShowPriceTax(),
				'customers_status_add_tax_ot'          => $customerGroup->getSettings()->isAddTaxOt(),
				'customers_status_payment_unallowed'   => implode(',', $customerGroup->getConfigurations()
				                                                                     ->getUnallowedPaymentModules()),
				'customers_status_shipping_unallowed'  => implode(',', $customerGroup->getConfigurations()
				                                                                     ->getUnallowedShippingModules()),
				'customers_status_discount_attributes' => $customerGroup->getSettings()->isDiscountAttributes(),
				'customers_fsk18'                      => $customerGroup->getSettings()->isFsk18(),
				'customers_fsk18_display'              => $customerGroup->getSettings()->isFsk18Display(),
				'customers_status_write_reviews'       => $customerGroup->getSettings()->isWriteReviews(),
				'customers_status_read_reviews'        => $customerGroup->getSettings()->isReadReviews()
			], ['customers_status_id' => $customerGroup->getId(), 'language_id' => $languageId]);
		}
		
		return $this;
	}
}