<?php
/* --------------------------------------------------------------
   UpdateDownloader.inc.php 2018-01-15
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

MainFactory::load_class('UpdateDownloaderInterface');

/**
 * Class UpdateDownloader
 */
class UpdateDownloader implements UpdateDownloaderInterface
{
	/**
	 * @var string
	 */
	private $hubConnectorVersion;
	
	/**
	 * @var CI_DB_query_builder
	 */
	private $db;
	
	/**
	 * @var string
	 */
	private $updateDownloaderVersion;
	
	/**
	 * @var bool
	 */
	private $downloadBeforeUpdate;
	
	/**
	 * @var bool
	 */
	private $downloadSuccessNotification;
	
	/**
	 * @var array
	 */
	private $downloadedUpdates;
	
	/**
	 * @var string
	 */
	private $gmKeyVersion;
	
	/**
	 * @var string
	 */
	private $gmKeyURL;
	
	/**
	 * @var messageStack_ORIGIN
	 */
	private $messageStack;
	
	/**
	 * @var LanguageTextManager
	 */
	private $textPhrases;
	
	/**
	 * @var string
	 */
	private $backupsDirectory;
	
	/**
	 * @var string
	 */
	private $updatesDirectory;
	
	/**
	 * @var string
	 */
	private $updateServer;
	
	/**
	 * @var array
	 */
	private $updateStatus;
	
	/**
	 * @var string
	 */
	private $langCode;
	
	
	/**
	 * UpdateDownloader constructor.
	 */
	public function __construct()
	{
		$this->db                          = StaticGXCoreLoader::getDatabaseQueryBuilder();
		$this->downloadBeforeUpdate        = true; // do not change!
		$this->downloadSuccessNotification = false; // do not change!
		$this->updateDownloaderVersion     = $this->getUpdateDownloaderVersion();
		$this->textPhrases                 = MainFactory::create('LanguageTextManager', 'update_downloader',
		                                                         $_SESSION['languages_id']);
		$this->backupsDirectory            = DIR_FS_CATALOG . 'cache' . DIRECTORY_SEPARATOR
		                                     . 'update_downloader/backups';
		$this->updatesDirectory            = DIR_FS_CATALOG . 'cache' . DIRECTORY_SEPARATOR
		                                     . 'update_downloader/updates';
		$this->messageStack                = $GLOBALS['messageStack'];
		$this->updateServer                = gm_get_conf('UPDATE_DOWNLOADER_URL');
		
		$languageProvider = MainFactory::create('LanguageProvider', $this->db);
		$this->langCode   = strtolower($languageProvider->getCodeById(new IdType((int)$_SESSION['languages_id'])));
		
		if(empty($this->updateServer))
		{
			$this->updateServer = 'https://updates.gambio-support.de/v1/check.php';
		}
		
		try
		{
			$this->checkUpdates();
			$this->checkDownloadedUpdates();
		}
		catch(UpdateDownloaderServerNotAvailableException $e)
		{
			$this->messageStack->add($e->getMessage(), 'info');
		}
	}
	
	
	/**
	 * Returns the current version of the Gambio Update Downloader.
	 *
	 * @return string
	 */
	public function getUpdateDownloaderVersion()
	{
		if(!empty($this->updateDownloaderVersion))
		{
			return $this->updateDownloaderVersion;
		}
		
		$versions = $this->db->select('version')
		                     ->from('version_history')
		                     ->where('name LIKE "Update Downloader v%"')
		                     ->order_by('version', 'DESC')
		                     ->get()
		                     ->result();
		
		if(count($versions) > 0 && !empty($versions[0]->version))
		{
			return $versions[0]->version;
		}
		
		return '0.0.0';
	}
	
	
	/**
	 * Returns the current value of the "download before update" option.
	 *
	 * @return bool
	 */
	public function getDownloadBeforeUpdate()
	{
		return $this->downloadBeforeUpdate;
	}
	
	
	/**
	 * Returns the update status as an array. The array contains the keys body and info, which contains the body and
	 * header information of the curl request that was made.
	 *
	 * @return array
	 */
	public function getUpdatesStatus()
	{
		return $this->updateStatus;
	}
	
	
	/**
	 * Returns an arraywith all update versions which are already downloaded.
	 *
	 * @return array
	 */
	public function getDownloadedUpdates()
	{
		return $this->downloadedUpdates;
	}
	
	
	/**
	 * Starts the download of all available updates.
	 *
	 * @throws \UpdateDownloaderDownloadFailedException
	 * @throws \UpdateDownloaderNoUpdateAvailableException
	 */
	public function downloadUpdates()
	{
		$this->deleteUpdatesDirectory();
		
		if(count($this->updateStatus['body']['updates']) === 0)
		{
			throw new UpdateDownloaderNoUpdateAvailableException($this->textPhrases->get_text('NO_UPDATES'));
		}
		
		if(!class_exists('ZipArchive'))
		{
			$this->messageStack->add_session($this->textPhrases->get_text('ERROR_ZIPCLASS_MISSING'), 'notice');
			
			foreach($this->updateStatus['body']['updates'] as $updateKey => $update)
			{
				try
				{
					$this->downloadUpdateNoZip($update, $updateKey);
				}
				catch(Exception $e)
				{
					if($e instanceof UpdateDownloaderUpdateNotAvailableException === false
					   && $e instanceof UpdateDownloaderMD5HashException === false
					   && $e instanceof UpdateDownloaderNoPermissionException === false)
					{
						# catch only own exceptions
						throw $e;
					}
					elseif($e instanceof UpdateDownloaderNoPermissionException)
					{
						throw new UpdateDownloaderDownloadFailedException(sprintf($this->textPhrases->get_text('UPDATE_NOT_AVAILABLE'),
						                                                          $update['name']['en']));
					}
					
					$this->messageStack->add_session($e->getMessage(), 'error');
					throw new UpdateDownloaderDownloadFailedException();
				}
			}
			
			$this->checkDownloadedUpdates();
			
			return;
		}
		
		foreach($this->updateStatus['body']['updates'] as $updateKey => $update)
		{
			try
			{
				$this->downloadUpdateZip($update, $updateKey);
			}
			catch(Exception $e)
			{
				if($e instanceof UpdateDownloaderUpdateNotAvailableException === false
				   && $e instanceof UpdateDownloaderMD5HashException === false
				   && $e instanceof UpdateDownloaderNoPermissionException === false)
				{
					# catch only own exceptions
					throw $e;
				}
				elseif($e instanceof UpdateDownloaderNoPermissionException)
				{
					throw new UpdateDownloaderDownloadFailedException(sprintf($this->textPhrases->get_text('UPDATE_NOT_AVAILABLE'),
					                                                          $update['name']['en']));
				}
				
				$this->messageStack->add_session($e->getMessage(), 'error');
				throw new UpdateDownloaderDownloadFailedException();
			}
		}
		
		$this->checkDownloadedUpdates();
		
		return;
	}
	
	
	/**
	 * Starts the installation of all available downloads. Updates will be downloaded bevore installation, if the
	 * "download before update" option is active.
	 *
	 * @throws \UpdateDownloaderDownloadFailedException
	 * @throws \UpdateDownloaderNoInitialPointException
	 * @throws \UpdateDownloaderNoUpdateAvailableException
	 * @throws \UpdateDownloaderUpdateFailedException
	 * @throws \UpdateDownloaderUpdatesNotDownloadedException
	 */
	public function installUpdates()
	{
		if(count($this->updateStatus['body']['updates']) === 0)
		{
			throw new UpdateDownloaderNoUpdateAvailableException($this->textPhrases->get_text('NO_UPDATES'));
		}
		
		if($this->downloadBeforeUpdate)
		{
			$this->downloadUpdates();
		}
		
		if(count($this->updateStatus['body']['updates']) !== count($this->downloadedUpdates))
		{
			throw new UpdateDownloaderUpdatesNotDownloadedException($this->textPhrases->get_text('UPDATES_NOT_DOWNLOADED'));
		}
		
		if(count($this->updateStatus['body']['updates']) > 0)
		{
			foreach($this->updateStatus['body']['updates'] as $updateKey => $update)
			{
				try
				{
					$this->checkPermission($updateKey);
					$this->backupFiles($updateKey);
					$this->copyFiles($updateKey);
					
					$this->messageStack->add_session(sprintf($this->textPhrases->get_text('UPDATE_SUCCESS'),
					                                         $update['name']['en']), 'success');
				}
				catch(Exception $e)
				{
					if($e instanceof UpdateDownloaderNoPermissionException === false
					   && $e instanceof UpdateDownloaderBackupCreationFailedException === false
					   && $e instanceof UpdateDownloaderCopyFailedException === false)
					{
						# throw all exceptions that doesn't come from updater
						throw $e;
					}
					
					if($e instanceof UpdateDownloaderNoPermissionException === false)
					{
						$this->messageStack->add_session($e->getMessage(), 'error');
					}
					
					$this->messageStack->add_session(sprintf($this->textPhrases->get_text('IMPORT_BACKUP'),
					                                         $update['name']['en']), 'info');
					
					$this->restoreBackup($updateKey);
					$this->deleteBackupsDirectory();
					$this->deleteUpdatesDirectory();
					
					throw new UpdateDownloaderUpdateFailedException(sprintf($this->textPhrases->get_text('UPDATE_FAILED'),
					                                                        $update['name']['en']));
				}
			}
			
			$this->deleteBackupsDirectory();
			$this->deleteUpdatesDirectory();
		}
		else
		{
			throw new UpdateDownloaderNoUpdateAvailableException($this->textPhrases->get_text('NO_UPDATES'));
		}
	}
	
	
	/**
	 * Checks the update server for new updates. Saves the result internally.
	 *
	 * @throws \UpdateDownloaderServerNotAvailableException
	 */
	protected function checkUpdates()
	{
		$this->updateStatus = [];
		$options            = [
			CURLOPT_CONNECTTIMEOUT => 10,
			CURLOPT_POST           => true,
			CURLOPT_POSTFIELDS     => [
				'shopVersion'     => gm_get_conf('INSTALLED_VERSION'),
				'shopUrl'         => HTTP_SERVER . DIR_WS_CATALOG,
				'shopKey'         => $this->getShopKey(),
				'versionHistory'  => json_encode($this->getVersionHistory()),
				'versionReceipts' => json_encode($this->getVersionReceipts()),
			],
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_URL            => $this->updateServer,
		];
		
		$curlHandle = curl_init();
		curl_setopt_array($curlHandle, $options);
		$this->updateStatus['body'] = curl_exec($curlHandle);
		$this->updateStatus['body'] = json_decode($this->updateStatus['body'], true);
		$this->updateStatus['info'] = curl_getinfo($curlHandle);
		curl_close($curlHandle);
		
		if($this->updateStatus['info']['http_code'] !== 200)
		{
			throw new UpdateDownloaderServerNotAvailableException($this->textPhrases->get_text('NO_UPDATES'));
		}
	}
	
	
	/**
	 * Checks the updates directory for downloaded updates. Saves the result internally.
	 */
	protected function checkDownloadedUpdates()
	{
		$this->downloadedUpdates = [];
		
		if(file_exists($this->updatesDirectory) && is_dir($this->updatesDirectory)
		   && count($this->updateStatus['body']['updates']) > 0)
		{
			foreach($this->updateStatus['body']['updates'] as $updateKey => $update)
			{
				$updateDir = $this->updatesDirectory . DIRECTORY_SEPARATOR . $updateKey;
				if(file_exists($updateDir) && is_dir($updateDir))
				{
					$this->downloadedUpdates[] = $updateKey;
				}
			}
		}
	}
	
	
	/**
	 * Sets the Gambio Update Downloader version to the given value.
	 *
	 * @param string $version
	 */
	protected function setUpdateDownloaderVersion($version)
	{
		$this->updateDownloaderVersion = $version;
	}
	
	
	/**
	 * Downloads a single update als a zip archiv and unpacks it into in the updates directory.
	 * If the zip extension is not available on the webserver, the downloadUpdateNoZip method will be used as a
	 * fallback.
	 *
	 * @param array $update
	 *
	 * @throws \UpdateDownloaderNoPermissionException
	 * @throws \UpdateDownloaderMD5HashException
	 */
	protected function downloadUpdateZip($update, $updateKey)
	{
		if(!file_exists($this->updatesDirectory) || !is_dir($this->updatesDirectory))
		{
			$createDir = @mkdir($this->updatesDirectory, 0777, true);
			if($createDir === false)
			{
				$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_DIR'),
				                                         $this->updatesDirectory), 'error');
				throw new UpdateDownloaderNoPermissionException();
			}
		}
		
		$curl = curl_init($update['zip']);
		curl_setopt($curl, CURLOPT_NOBODY, true);
		curl_exec($curl);
		$fileExists = curl_getinfo($curl, CURLINFO_HTTP_CODE) === 200;
		curl_close($curl);
		
		if(!$fileExists)
		{
			$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_ZIPFILE_MISSING'),
			                                         $update['name'][$this->langCode]), 'notice');
			$this->downloadUpdateNoZip($update, $updateKey);
		}
		else
		{
			$zipFile    = $this->updatesDirectory . DIRECTORY_SEPARATOR . $updateKey . '.zip';
			$createFile = file_put_contents($zipFile, fopen($update['zip'], 'r'));
			
			if($createFile === false)
			{
				$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_FILE'),
				                                         $zipFile), 'error');
				throw new UpdateDownloaderNoPermissionException();
			}
			
			try
			{
				if(isset($update['zip_hash']) && md5_file($zipFile) !== $update['zip_hash'])
				{
					throw new UpdateDownloaderMD5HashException(sprintf($this->textPhrases->get_text('ERROR_MD5_HASH'),
					                                                   $zipFile));
				}
				
				$this->unzipUpdate($update, $updateKey);
				@unlink($zipFile);
				
				if($this->downloadSuccessNotification)
				{
					$this->messageStack->add_session(sprintf($this->textPhrases->get_text('UPDATE_DOWNLOADED'),
					                                         $update['name'][$this->langCode]), 'success');
				}
			}
			catch(Exception $e)
			{
				if($e instanceof UpdateDownloaderZipCorruptedException === false
				   && $e instanceof UpdateDownloaderMD5HashException === false)
				{
					# throw all exceptions that doesn't come from updater
					throw $e;
				}
				
				$this->messageStack->add_session($e->getMessage(), 'notice');
				@unlink($zipFile);
				$this->downloadUpdateNoZip($update, $updateKey);
			}
		}
	}
	
	
	/**
	 * Downloads all update files from a single update into in the updates directory.
	 * This method is a fallback, if the zip extension is not available on the webserver.
	 *
	 * @param array $update
	 *
	 * @throws \UpdateDownloaderNoPermissionException
	 * @throws \UpdateDownloaderUpdateNotAvailableException
	 * @throws \UpdateDownloaderMD5HashException
	 */
	protected function downloadUpdateNoZip($update, $updateKey)
	{
		$updateDir = $this->updatesDirectory . DIRECTORY_SEPARATOR . $updateKey;
		
		if(!file_exists($updateDir) || !is_dir($updateDir))
		{
			$createDir = @mkdir($updateDir, 0777, true);
			if($createDir === false)
			{
				$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_DIR'),
				                                         $updateDir), 'error');
				throw new UpdateDownloaderNoPermissionException();
			}
		}
		
		if(count($update['filelist']) === 0)
		{
			$this->deleteDirectory($updateDir);
			$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_FILELIST_EMPTY'),
			                                         $update['name'][$this->langCode]), 'notice');
			throw new UpdateDownloaderUpdateNotAvailableException(sprintf($this->textPhrases->get_text('UPDATE_NOT_AVAILABLE'),
			                                                              $update['name'][$this->langCode]));
		}
		
		foreach($update['filelist'] as $updateFile)
		{
			$curl = curl_init($updateFile['source']);
			curl_setopt($curl, CURLOPT_NOBODY, true);
			curl_exec($curl);
			$fileExists = curl_getinfo($curl, CURLINFO_HTTP_CODE) === 200;
			curl_close($curl);
			
			if(!$fileExists)
			{
				$this->deleteDirectory($updateDir);
				throw new UpdateDownloaderUpdateNotAvailableException(sprintf($this->textPhrases->get_text('UPDATE_NOT_AVAILABLE'),
				                                                              $update['name'][$this->langCode]));
			}
			else
			{
				$updateFileDestination    = $updateDir . DIRECTORY_SEPARATOR . $updateFile['destination'];
				$updateFileDestinationDir = substr($updateFileDestination, 0,
				                                   strrpos($updateFileDestination, DIRECTORY_SEPARATOR));
				
				if(!file_exists($updateFileDestinationDir) || !is_dir($updateFileDestinationDir))
				{
					$createDir = @mkdir($updateFileDestinationDir, 0777, true);
					if($createDir === false)
					{
						$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_DIR'),
						                                         $updateFileDestinationDir), 'error');
						throw new UpdateDownloaderNoPermissionException();
					}
				}
				
				$createFile = file_put_contents($updateFileDestination, fopen($updateFile['source'], 'r'));
				if($createFile === false)
				{
					$this->deleteUpdatesDirectory();
					$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_FILE'),
					                                         $updateFileDestination), 'error');
					throw new UpdateDownloaderNoPermissionException();
				}
				elseif(isset($updateFile['hash']) && md5_file($updateFileDestination) !== $updateFile['hash'])
				{
					$this->deleteUpdatesDirectory();
					throw new UpdateDownloaderMD5HashException(sprintf($this->textPhrases->get_text('ERROR_MD5_HASH'),
					                                                   $updateFileDestination));
				}
			}
		}
		
		if($this->downloadSuccessNotification)
		{
			$this->messageStack->add_session(sprintf($this->textPhrases->get_text('UPDATE_DOWNLOADED'),
			                                         $update['name'][$this->langCode]), 'success');
		}
	}
	
	
	/**
	 * Unpacks the zip archiv of a downloaded update.
	 *
	 * @param array $update
	 *
	 * @throws \UpdateDownloaderNoPermissionException
	 * @throws \UpdateDownloaderZipCorruptedException
	 */
	protected function unzipUpdate($update, $updateKey)
	{
		$zipName   = $updateKey . '.zip';
		$updateDir = $this->updatesDirectory . DIRECTORY_SEPARATOR . $updateKey;
		
		if(file_exists($updateDir) && is_dir($updateDir))
		{
			$deleteDir = $this->deleteDirectory($updateDir);
			
			if($deleteDir === false)
			{
				$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_DIR'),
				                                         $updateDir), 'error');
				throw new UpdateDownloaderNoPermissionException();
			}
		}
		
		$createDir = @mkdir($updateDir, 0777, true);
		if($createDir === false)
		{
			$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_DIR'), $updateDir),
			                                 'error');
			throw new UpdateDownloaderNoPermissionException();
		}
		
		$zip          = new ZipArchive;
		$zipOpen      = @$zip->open($this->updatesDirectory . DIRECTORY_SEPARATOR . $zipName);
		$zipExtracted = @$zip->extractTo($updateDir);
		$zipClosed    = @$zip->close();
		if($zipOpen !== true || $zipExtracted === false || $zipClosed === false)
		{
			$this->deleteDirectory($updateDir);
			throw new UpdateDownloaderZipCorruptedException(sprintf($this->textPhrases->get_text('ERROR_ZIP_CORRUPTED'),
			                                                        $update['name'][$this->langCode]));
		}
	}
	
	
	/**
	 * Checks the permission (creation and writing of files and directories) for an update.
	 *
	 * @param $updateKey
	 *
	 * @throws \UpdateDownloaderNoPermissionException
	 */
	protected function checkPermission($updateKey)
	{
		$updateDir       = $this->updatesDirectory . DIRECTORY_SEPARATOR . $updateKey . DIRECTORY_SEPARATOR;
		$updateFiles     = $this->getDirectoryFiles($updateDir);
		$permissionError = [];
		
		if(count($updateFiles) > 0)
		{
			foreach($updateFiles as $updateFile)
			{
				$shopTestFile = str_replace($updateDir, DIR_FS_CATALOG, $updateFile . '.permission_check');
				$shopTestDir  = substr($shopTestFile, 0, strrpos($shopTestFile, DIRECTORY_SEPARATOR));
				
				if(!file_exists($shopTestDir) || !is_dir($shopTestDir))
				{
					$dirCreated = @mkdir($shopTestDir, 0777, true);
					
					if($dirCreated === false)
					{
						if(!in_array($shopTestDir, $permissionError))
						{
							$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_DIR'),
							                                         $shopTestDir), 'error');
						}
						$permissionError[] = $shopTestDir;
						continue;
					}
				}
				elseif(file_exists($shopTestDir) && is_dir($shopTestDir) && !is_writeable($shopTestDir))
				{
					if(!in_array($shopTestDir, $permissionError))
					{
						$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_DIR'),
						                                         $shopTestDir), 'error');
					}
					$permissionError[] = $shopTestDir;
					continue;
				}
				
				$fileOpen    = @fopen($shopTestFile, 'w');
				$fileWritten = @fwrite($fileOpen, 'permission test');
				$fileClosed  = @fclose($fileOpen);
				if($fileOpen === false || $fileWritten === false || $fileClosed === false)
				{
					if(!in_array($shopTestFile, $permissionError))
					{
						$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_FILE'),
						                                         $shopTestFile), 'error');
					}
					$permissionError[] = $shopTestFile;
					continue;
				}
				
				@unlink($shopTestFile);
			}
		}
		
		if(count($permissionError) > 0)
		{
			print_r($this->messageStack);
			throw new UpdateDownloaderNoPermissionException();
		}
	}
	
	
	/**
	 * Creates a backup.
	 *
	 * @param $updateKey
	 *
	 * @throws \UpdateDownloaderBackupCreationFailedException
	 * @throws \UpdateDownloaderNoPermissionException
	 */
	protected function backupFiles($updateKey)
	{
		$this->deleteBackupsDirectory();
		
		$backupDir   = $this->backupsDirectory . DIRECTORY_SEPARATOR . $updateKey . DIRECTORY_SEPARATOR;
		$updateDir   = $this->updatesDirectory . DIRECTORY_SEPARATOR . $updateKey . DIRECTORY_SEPARATOR;
		$updateFiles = $this->getDirectoryFiles($updateDir);
		
		if(!file_exists($backupDir) || !is_dir($backupDir))
		{
			$dirCreated = @mkdir($backupDir, 0777, true);
			if($dirCreated === false)
			{
				$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_DIR'),
				                                         $backupDir), 'error');
				throw new UpdateDownloaderNoPermissionException();
			}
		}
		
		if(count($updateFiles) > 0)
		{
			foreach($updateFiles as $updateFile)
			{
				$shopFile      = str_replace($updateDir, DIR_FS_CATALOG, $updateFile);
				$backupFile    = str_replace($updateDir, $backupDir, $updateFile);
				$backupFileDir = substr($backupFile, 0, strrpos($backupFile, DIRECTORY_SEPARATOR));
				
				if(file_exists($shopFile))
				{
					
					if(!file_exists($backupFileDir) || !is_dir($backupFileDir))
					{
						@mkdir($backupFileDir, 0777, true);
					}
					
					$fileCopied = @copy($shopFile, $backupFile);
					
					if(!$fileCopied)
					{
						$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_FILE'),
						                                         $backupFile), 'error');
						throw new UpdateDownloaderNoPermissionException();
					}
					elseif(filesize($shopFile) !== filesize($backupFile))
					{
						throw new UpdateDownloaderBackupCreationFailedException(sprintf($this->textPhrases->get_text('ERROR_BACKUP_CREATION_FAILED'),
						                                                                $shopFile));
					}
				}
			}
		}
	}
	
	
	/**
	 * Restores a backup.
	 *
	 * @param $updateVersion
	 *
	 * @throws \UpdateDownloaderBackupRestoreFailedException
	 */
	protected function restoreBackup($updateKey)
	{
		$backupDir   = $this->backupsDirectory . DIRECTORY_SEPARATOR . $updateKey . DIRECTORY_SEPARATOR;
		$updateDir   = $this->updatesDirectory . DIRECTORY_SEPARATOR . $updateKey . DIRECTORY_SEPARATOR;
		$updateFiles = $this->getDirectoryFiles($updateDir);
		
		if(count($updateFiles) > 0)
		{
			foreach($updateFiles as $updateFile)
			{
				$shopFile   = str_replace($updateDir, DIR_FS_CATALOG, $updateFile);
				$backupFile = str_replace($updateDir, $backupDir, $updateFile);
				
				if(file_exists($backupFile) && file_exists($shopFile))
				{
					$deleted = @unlink($shopFile);
					if($deleted === false)
					{
						throw new UpdateDownloaderBackupRestoreFailedException(sprintf($this->textPhrases->get_text('ERROR_BACKUP_RESTORE_FAILED'),
						                                                               $updateKey));
					}
					
					$fileCopied = @copy($backupFile, $shopFile);
					if(!$fileCopied || filesize($shopFile) !== filesize($backupFile))
					{
						throw new UpdateDownloaderBackupRestoreFailedException(sprintf($this->textPhrases->get_text('ERROR_BACKUP_RESTORE_FAILED'),
						                                                               $updateKey));
					}
				}
			}
		}
	}
	
	
	/**
	 * Copies the update files of a single update into the shop.
	 *
	 * @param $updateKey
	 *
	 * @throws \UpdateDownloaderNoPermissionException
	 * @throws \UpdateDownloaderCopyFailedException
	 */
	protected function copyFiles($updateKey)
	{
		$updateDir   = $this->updatesDirectory . DIRECTORY_SEPARATOR . $updateKey . DIRECTORY_SEPARATOR;
		$updateFiles = $this->getDirectoryFiles($updateDir);
		
		if(count($updateFiles) > 0)
		{
			foreach($updateFiles as $updateFile)
			{
				$shopFile = str_replace($updateDir, DIR_FS_CATALOG, $updateFile);
				$shopDir  = substr($shopFile, 0, strrpos($shopFile, DIRECTORY_SEPARATOR));
				
				if(!file_exists($shopDir) || !is_dir($shopDir))
				{
					@mkdir($shopDir, 0777, true);
				}
				
				@unlink($shopFile);
				$fileCopied = @copy($updateFile, $shopFile);
				if(!$fileCopied)
				{
					$this->messageStack->add_session(sprintf($this->textPhrases->get_text('ERROR_PERMISSON_FILE'),
					                                         $shopFile), 'error');
					throw new UpdateDownloaderNoPermissionException();
				}
				elseif(filesize($updateFile) !== filesize($shopFile))
				{
					throw new UpdateDownloaderCopyFailedException(sprintf($this->textPhrases->get_text('ERROR_COPY_FAILED'),
					                                                      $updateFile, $shopFile));
				}
			}
		}
	}
	
	
	/**
	 * Returns an array with all files of a directory and his subdirectories.
	 *
	 * @param string $dir
	 *
	 * @return array
	 */
	protected function getDirectoryFiles($dir)
	{
		$return = [];
		$files  = scandir($dir);
		foreach($files as $file)
		{
			if($file !== '.' && $file !== '..')
			{
				$path = realpath($dir . DIRECTORY_SEPARATOR . $file);
				if(!is_dir($path))
				{
					$return[] = $path;
				}
				else
				{
					$return = array_merge($return, $this->getDirectoryFiles($path));
				}
			}
		}
		
		return $return;
	}
	
	
	/**
	 * Deletes a directory. Return true on success and false otherwise.
	 *
	 * @param $dir
	 *
	 * @return bool
	 */
	protected function deleteDirectory($dir)
	{
		if(!file_exists($dir))
		{
			return true;
		}
		
		$files = array_diff(scandir($dir), ['.', '..']);
		foreach($files as $file)
		{
			if(is_dir($dir . DIRECTORY_SEPARATOR . $file))
			{
				$deleted = $this->deleteDirectory($dir . DIRECTORY_SEPARATOR . $file);
				if($deleted === false)
				{
					return false;
				}
			}
			else
			{
				$deleted = @unlink($dir . DIRECTORY_SEPARATOR . $file);
				if($deleted === false)
				{
					return false;
				}
			}
		}
		
		return rmdir($dir);
	}
	
	
	/**
	 * Deletes the backups directory. Return true on success and false otherwise.
	 *
	 * @return bool
	 */
	protected function deleteBackupsDirectory()
	{
		return $this->deleteDirectory($this->backupsDirectory);
	}
	
	
	/**
	 * Deletes the updates directory. Return true on success and false otherwise.
	 *
	 * @return bool
	 */
	protected function deleteUpdatesDirectory()
	{
		return $this->deleteDirectory($this->updatesDirectory);
	}
	
	
	/**
	 * Returns the shop key from database.
	 *
	 * @return mixed
	 */
	protected function getShopKey()
	{
		$query   = $this->db->select('configuration_value')
		                    ->from('configuration')
		                    ->where('configuration_key', 'GAMBIO_SHOP_KEY')
		                    ->get();
		$shopKey = $query->row();
		if(isset($shopKey->configuration_value))
		{
			return $shopKey->configuration_value;
		}
		
		return null;
	}
	
	
	/**
	 * Returns the version history from database.
	 *
	 * @return mixed
	 */
	protected function getVersionHistory()
	{
		$dbData = $this->db->select()->from('version_history')->get()->result_array();
		
		return $dbData;
	}
	
	
	/**
	 * Returns the version receipts from filesystem.
	 *
	 * @return mixed
	 */
	protected function getVersionReceipts()
	{
		return scandir(DIR_FS_CATALOG . 'version_info');
	}
}